;;; Experimental functions for planner.el

(require 'planner)

;; Testing arch 
;;;_* Notes

;;;_ + Linking through emacs-wiki links

(defun planner-jump-to-linked-note (&optional note-info)
  "Display the note linked to by the current note or NOTE-INFO if non-nil."
  (interactive)
  (setq note-info (or note-info (planner-current-note-info)))
  (when (planner-note-link note-info)
    (emacs-wiki-visit-link (planner-note-link note-info))
    (widen)
    t))

(defun planner-update-note ()
  "Copy the text from this note to the linked note, if any."
  (interactive)
  (when (planner-narrow-to-note)
    (let ((body (buffer-substring-no-properties (point-min) (point-max)))
          (info (planner-current-note-info)))
      ;; Get rid of the anchor.
      (when (string-match "^\\.#[0-9]+\\s-+" body)
        (setq body (replace-match "" nil t body)))
      ;; Get rid of the link
      (when (string-match "\\s-+(\\[\\[.+?\\]\\])" body)
        (setq body (replace-match "" nil t body)))
      ;; Insert the new body
      (when (planner-jump-to-linked-note)
        (when (planner-narrow-to-note)
          (goto-char (point-min))
          (skip-chars-forward ".#0-9")
          (delete-region (point) (point-max))
          (insert " " body)
          (goto-char (point-min))
          (goto-char (line-end-position))
          (insert " ([[" (planner-note-page info) "#" (planner-note-anchor info) "]])"))))))

;;;_ + Searching

;;;_* Tasks 

(defun planner-remove-duplicates ()
  (interactive)
  (goto-char (point-min))
  (while (re-search-forward "^#[A-C][0-9]*\\s-+\\(.+\\)$" nil t)
    (save-excursion
      (let ((text (match-string-no-properties 1)))
        (while (re-search-forward (concat "^#[A-C][0-9]*\\s-+"
                                          (regexp-quote text)
                                          "$") nil t)
          (delete-region (line-beginning-position) (1+ (line-end-position))))))))

;;;_* Indexing
;; I want to compress the list of day pages. Arranging them by month
;; may be a good strategy, although a calendar would be optimal.

(defadvice emacs-wiki-generate-index (around planner-experimental activate)
  (setq ad-return-value (if (equal major-mode 'planner-mode)
                            (planner-generate-index (ad-get-arg 0)
                                                    (ad-get-arg 1))
                          ad-do-it)))

(defun planner-index ()
  "Display an index of all known Wiki pages."
  (interactive)
  (message "Generating Wiki index...")
  (pop-to-buffer (planner-generate-index))
  (goto-char (point-min))
  (planner-mode)
  (message "Generating Wiki index...done"))

(defun planner-generate-index (&optional as-list exclude-private)
  "Generate an index of all Wiki pages."
  (let ((project emacs-wiki-current-project))
    (with-current-buffer (get-buffer-create "*Wiki Index*")
      (erase-buffer)
      (if project
          (emacs-wiki-change-project project))
      (let ((files (sort (copy-alist (emacs-wiki-file-alist))
                         (function
                          (lambda (l r)
                            (string-lessp (car l) (car r))))))
            file
            dates
            month
            last-month)
        (while files
          (unless (and exclude-private
                       (emacs-wiki-private-p (caar files)))
            (if (string-match planner-date-regexp (caar files))
                (setq dates (append (list (caar files)) dates))
              (insert (if as-list "- " "") "[[" (caar files) "]]\n")))
          (setq files (cdr files)))
        (when dates
          (insert "\n")
          (while dates
            (setq month (substring (car dates) 0 7))
            (unless (equal month last-month)
              (setq last-month month)
              (insert "\n" month " |"))
            (insert " [[" (car dates) "][" (substring (car dates) 8) "]]")
            (setq dates (cdr dates))))
      (current-buffer)))))


;;;_* Local emacs vars.
;;;Local variables:
;;;allout-layout: (-1 0 : )
;;;End:
